package org.jboss.qa.drools.backwardchaining;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.type.Position;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;

public class BackwardChaining {

    public static void main(String[] args) {
        KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
        kbuilder.add(ResourceFactory.newClassPathResource("org/jboss/qa/drools/backwardchaining/backwardchaining.drl"), ResourceType.DRL);

        if (kbuilder.hasErrors()) {
            for (KnowledgeBuilderError error : kbuilder.getErrors()) {
                System.err.println(error);
            }
            throw new RuntimeException();
        }

        KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
        kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());

        StatefulKnowledgeSession ksession = kbase.newStatefulKnowledgeSession();

        System.out.println();
        System.out.println("Run#1 - insert place first");
        System.out.println();

        ksession.insert("kitchen");

        ksession.insert(new Locator("apple", "kitchen"));
        ksession.insert(new Locator("table", "kitchen"));
        ksession.insert(new Locator("desk", "office"));
        ksession.insert(new Locator("fork", "table"));
        ksession.insert(new Locator("box", "table"));
        ksession.insert(new Locator("sweets", "box"));

        ksession.fireAllRules();

        ksession.dispose();

        ksession = kbase.newStatefulKnowledgeSession();

        System.out.println();
        System.out.println("Run#2 - insert place last");
        System.out.println();

        ksession.insert(new Locator("apple", "kitchen"));
        ksession.insert(new Locator("table", "kitchen"));
        ksession.insert(new Locator("desk", "office"));
        ksession.insert(new Locator("fork", "table"));
        ksession.insert(new Locator("box", "table"));
        ksession.insert(new Locator("sweets", "box"));

        ksession.insert("kitchen");

        ksession.fireAllRules();

        ksession.dispose();
    }

    public static class Locator {
        @Position(0)
        private String thing;
        @Position(1)
        private String place;

        public Locator() {}

        public Locator(String thing, String place) {
            this.thing = thing;
            this.place = place;
        }

        public String getThing() {
            return thing;
        }
        public void setThing(String thing) {
            this.thing = thing;
        }
        public String getPlace() {
            return place;
        }
        public void setPlace(String place) {
            this.place = place;
        }


    }
}
